@@ -4,7 +4,6 @@ import subprocess
 import numpy as np
 from natsort import natsorted
 from tqdm import tqdm
-
 from cellpose import utils, models, io
 
 try:
@@ -19,7 +18,7 @@ except Exception as err:
     GUI_ERROR = err
     GUI_IMPORT = False
     raise
-
+    
 import logging
 logger = logging.getLogger(__name__)
 
@@ -32,13 +31,23 @@ def confirm_prompt(question):
 def install(package):
     subprocess.check_call([sys.executable, "-m", "pip", "install", package])
     
+def check_omni(logger,omni=False):
+    if omni and not 'omnipose' not in sys.modules:
+        logger.info('Omnipose features requested but not installed.')
+        confirm = confirm_prompt('Install Omnipose?')
+        if confirm:
+            install('omnipose')
+        else:
+            logger.info('>>>> Omnipose not installed. Running with omni=False')
+        return confirm
+    
 # settings re-grouped a bit
 def main():
     parser = argparse.ArgumentParser(description='cellpose parameters')
     
     # settings for CPU vs GPU
     hardware_args = parser.add_argument_group("hardware arguments")
-    hardware_args.add_argument('--use_gpu', action='store_true', help='use gpu if mxnet with cuda installed')
+    hardware_args.add_argument('--use_gpu', action='store_true', help='use gpu if torch or mxnet with cuda installed')
     hardware_args.add_argument('--check_mkl', action='store_true', help='check if mkl working')
     hardware_args.add_argument('--mkldnn', action='store_true', help='for mxnet, force MXNET_SUBGRAPH_BACKEND = "MKLDNN"')
         
@@ -46,7 +55,7 @@ def main():
     input_img_args = parser.add_argument_group("input image arguments")
     input_img_args.add_argument('--dir',
                         default=[], type=str, help='folder containing data to run or train on.')
-    input_img_args.add_argument('--look_one_level_down', action='store_true', help='')
+    input_img_args.add_argument('--look_one_level_down', action='store_true', help='run processing on all subdirectories of current folder')
     input_img_args.add_argument('--mxnet', action='store_true', help='use mxnet')
     input_img_args.add_argument('--img_filter',
                         default=[], type=str, help='end string for images to run on')
@@ -93,7 +102,7 @@ def main():
     output_args = parser.add_argument_group("output arguments")
     output_args.add_argument('--save_png', action='store_true', help='save masks as png and outlines as text file for ImageJ')
     output_args.add_argument('--save_tif', action='store_true', help='save masks as tif and outlines as text file for ImageJ')
-    output_args.add_argument('--no_npy', action='store_false', help='suppress saving of npy')
+    output_args.add_argument('--no_npy', action='store_true', help='suppress saving of npy')
     output_args.add_argument('--savedir',
                         default=None, type=str, help='folder to which segmentation results will be saved (defaults to input image directory)')
     output_args.add_argument('--dir_above', action='store_true', help='save output folders adjacent to image folder instead of inside it (off by default)')
@@ -130,10 +139,10 @@ def main():
     # misc settings
     parser.add_argument('--verbose', action='store_true', help='flag to output extra information (e.g. diameter metrics) for debugging and fine-tuning parameters')
     parser.add_argument('--testing', action='store_true', help='flag to suppress CLI user confirmation for saving output; for test scripts')
-
-
-
+    
     args = parser.parse_args()
+
+
     
     # handle mxnet option 
     if args.check_mkl:
@@ -148,14 +157,22 @@ def main():
     
     if len(args.dir)==0:
         if not GUI_ENABLED:
-            logger.critical('ERROR: %s'%GUI_ERROR)
+            print('GUI ERROR: %s'%GUI_ERROR)
             if GUI_IMPORT:
-                logger.critical('GUI FAILED: GUI dependencies may not be installed, to install, run')
-                logger.critical('     pip install cellpose[gui]')
+                print('GUI FAILED: GUI dependencies may not be installed, to install, run')
+                print('     pip install cellpose[gui]')
         else:
             gui.run()
 
     else:
+        if args.verbose:
+            from .io import logger_setup
+            logger, log_file = logger_setup()
+        else:
+            print('>>>> !NEW LOGGING SETUP! To see cellpose progress, set --verbose')
+            print('No --verbose => no progress or info printed')
+            logger = logging.getLogger(__name__)
+
         use_gpu = False
         channels = [args.chan, args.chan2]
 
@@ -170,7 +187,7 @@ def main():
         if not (args.train or args.train_size):
             saving_something = args.save_png or args.save_tif or args.save_flows or args.save_ncolor or args.save_txt
             if not (saving_something or args.testing): 
-                logger.info('>>>> Running without saving any output.')
+                print('>>>> Running without saving any output.')
                 confirm = confirm_prompt('Proceed Anyway?')
                 if not confirm:
                     exit()
@@ -189,17 +206,18 @@ def main():
         if 'omni' in args.pretrained_model:
             args.omni = True
         
-        if args.omni:
-            logger.info('>>>> Omnipose enabled. See https://raw.githubusercontent.com/MouseLand/cellpose/master/cellpose/omnipose/license.txt for licensing details.')
-        
-        if (args.omni or args.cluster) and 'sklearn' not in sys.modules:
-            logger.info('>>>> Omnipose requires scikit-learn for DBSCAN clustering.')
+        if args.cluster and 'sklearn' not in sys.modules:
+            print('>>>> DBSCAN clustering requires scikit-learn.')
             confirm = confirm_prompt('Install scikit-learn?')
             if confirm:
                 install('scikit-learn')
             else:
-                logger.info('>>>> scikit-learn not installed. DBSCAN clustering will be automatically disabled.')
-                    
+                print('>>>> scikit-learn not installed. DBSCAN clustering will be automatically disabled.')
+                          
+        omni = check_omni(args.omni) # repeat the above check but factor it for use elsewhere
+        if args.omni:
+            print('>>>> Omnipose enabled. See https://raw.githubusercontent.com/MouseLand/cellpose/master/cellpose/omnipose/license.txt for licensing details.')
+        
         
         if not args.train and not args.train_size:
             tic = time.time()
@@ -221,7 +239,8 @@ def main():
             cstr1 = ['NONE', 'RED', 'GREEN', 'BLUE']
             logger.info('>>>> running cellpose on %d images using chan_to_seg %s and chan (opt) %s'%
                             (nimg, cstr0[channels[0]], cstr1[channels[1]]))
-            logger.info('>>>> omni is %d, cluster is %d'%(args.omni,args.cluster))
+            if args.omni:
+                logger.info('>>>> omni is ON, cluster is %d'%(args.omni,args.cluster))
              
             # handle built-in model exceptions; bacterial ones get no size model 
             if builtin_model:
