@@ -21,17 +21,16 @@ try:
     import torch
 #     from GPUtil import showUtilization as gpu_usage #for gpu memory debugging 
     from torch import nn
-    import torch_optimizer as optim # for RADAM optimizer
     from torch.utils import mkldnn as mkldnn_utils
     from . import resnet_torch
     TORCH_ENABLED = True
     torch_GPU = torch.device('cuda')
     torch_CPU = torch.device('cpu')
-except:
+except Exception as e:
     TORCH_ENABLED = False
+    print(e)
 
 core_logger = logging.getLogger(__name__)
-core_logger.setLevel(logging.DEBUG)
 tqdm_out = utils.TqdmToLogger(core_logger, level=logging.INFO)
 
 # nclasses now specified by user or by model type in models.py
@@ -41,12 +40,11 @@ def parse_model_string(pretrained_model):
     else:
         model_str = os.path.split(pretrained_model)[-1]
     if len(model_str)>3 and model_str[:4]=='unet':
-        core_logger.info(f'parsing model string {model_str} to get unet options')
         nclasses = max(2, int(model_str[4]))
     elif len(model_str)>7 and model_str[:8]=='cellpose':
-        core_logger.info(f'parsing model string {model_str} to get cellpose options')
+        nclasses = 3
     else:
-        return None
+        return True, True, False
     ostrs = model_str.split('_')[2::2]
     residual_on = ostrs[0]=='on'
     style_on = ostrs[1]=='on'
@@ -118,7 +116,7 @@ class UnetModel():
     def __init__(self, gpu=False, pretrained_model=False,
                     diam_mean=30., net_avg=True, device=None,
                     residual_on=False, style_on=False, concatenation=True,
-                    nclasses=4, torch=True, nchan=2):
+                    nclasses=3, torch=True, nchan=2):
         self.unet = True
         if torch:
             if not TORCH_ENABLED:
@@ -344,7 +342,10 @@ class UnetModel():
             self.net.eval()
             if self.mkldnn:
                 self.net = mkldnn_utils.to_mkldnn(self.net)
-        y, style = self.net(X)
+            with torch.no_grad():
+                y, style = self.net(X)
+        else:
+            y, style = self.net(X)
         if self.mkldnn:
             self.net.to(torch_CPU)
         y = self._from_device(y)
@@ -488,7 +489,6 @@ class UnetModel():
         # transpose so channels axis is last again
         y = np.transpose(y, detranspose)
         
-        torch.cuda.empty_cache() #release cuda memory
         return y, style
     
     def _run_tiled(self, imgi, augment=False, bsize=224, tile_overlap=0.1, return_conv=False):
@@ -681,7 +681,7 @@ class UnetModel():
 
     def train(self, train_data, train_labels, train_files=None, 
               test_data=None, test_labels=None, test_files=None,
-              channels=None, normalize=True, pretrained_model=None, save_path=None, save_every=50, save_each=False,
+              channels=None, normalize=True, save_path=None, save_every=50, save_each=False,
               learning_rate=0.2, n_epochs=500, momentum=0.9, weight_decay=0.00001, batch_size=8, rescale=False):
         """ train function uses 0-1 mask label and boundary pixels for training """
 
@@ -714,8 +714,7 @@ class UnetModel():
         del train_data[::8], train_classes[::8], train_labels[::8]
 
         model_path = self._train_net(train_data, train_classes, 
-                                     test_data, test_classes,
-                                     pretrained_model, save_path, save_every, save_each,
+                                     test_data, test_classes, save_path, save_every, save_each,
                                      learning_rate, n_epochs, momentum, weight_decay, 
                                      batch_size, rescale)
 
@@ -762,10 +761,10 @@ class UnetModel():
         X = self._to_device(x)
         if self.torch:
             self.optimizer.zero_grad()
-            if self.gpu:
-                self.net.train().cuda()
-            else:
-                self.net.train()
+            #if self.gpu:
+            #    self.net.train() #.cuda()
+            #else:
+            self.net.train()
             y = self.net(X)[0]
             loss = self.loss_fn(lbl,y)
             loss.backward()
@@ -785,10 +784,11 @@ class UnetModel():
         X = self._to_device(x)
         if self.torch:
             self.net.eval()
-            y, style = self.net(X)
-            loss = self.loss_fn(lbl,y)
-            test_loss = loss.item()
-            test_loss *= len(x)
+            with torch.no_grad():
+                y, style = self.net(X)
+                loss = self.loss_fn(lbl,y)
+                test_loss = loss.item()
+                test_loss *= len(x)
         else:
             y, style = self.net(X)
             loss = self.loss_fn(lbl, y)
@@ -797,23 +797,15 @@ class UnetModel():
 
     def _set_optimizer(self, learning_rate, momentum, weight_decay, SGD=False):
         if self.torch:
-        # best optimizer I tested seemed to be RAdam, about 2x as fast as SGD and very stable.
-        # Ranger21 is in beta and might be better/faster, but more testing is needed.
-        # Ranger21 has a convenient current_lr field, whereas RAdam doesn't and I just set this field to the learning rate
             if SGD:
                 self.optimizer = torch.optim.SGD(self.net.parameters(), lr=learning_rate,
                                             momentum=momentum, weight_decay=weight_decay)
             else:
+                import torch_optimizer as optim # for RADAM optimizer
                 self.optimizer = optim.RAdam(self.net.parameters(), lr=learning_rate, betas=(0.95, 0.999), #changed to .95
                                             eps=1e-08, weight_decay=weight_decay)
                 core_logger.info('>>> Using RAdam optimizer')
-#             self.optimizer = optim.AdaBound(self.net.parameters(), lr=learning_rate, betas=(0.9, 0.999), 
-#                                 gamma=1e-3, eps=1e-08, final_lr=0.15, weight_decay=weight_decay)
-#             print('>>> Using AdaBound optimizer')
-#             self.optimizer = Ranger21(self.net.parameters(), lr=learning_rate, weight_decay=weight_decay, num_batches_per_epoch=self.batch_size, 
-#                                       num_epochs=self.n_epochs,num_warmup_iterations=20*self.batch_size, betas=(0.9, 0.999), eps=1e-08)
-#             print('>>> Using Ranger21 optimizer')
-            self.optimizer.current_lr = learning_rate
+                self.optimizer.current_lr = learning_rate
         else:
             self.optimizer = gluon.Trainer(self.net.collect_params(), 'sgd',{'learning_rate': learning_rate,
                                 'momentum': momentum, 'wd': weight_decay})
@@ -848,9 +840,9 @@ class UnetModel():
     # Restored defaults. Need to make sure rescale is properly turned off and omni turned on when using CLI. 
     def _train_net(self, train_data, train_labels, 
               test_data=None, test_labels=None,
-              pretrained_model=None, save_path=None, save_every=100, save_each=False,
+              save_path=None, save_every=100, save_each=False,
               learning_rate=0.2, n_epochs=500, momentum=0.9, weight_decay=0.00001, 
-              SGD=False, batch_size=8, rescale=True, netstr='cellpose'): 
+              SGD=True, batch_size=8, rescale=True, netstr=None): 
         """ train function uses loss function self.loss_fn in models.py"""
         
         d = datetime.datetime.now()
@@ -876,13 +868,12 @@ class UnetModel():
 
         nchan = train_data[0].shape[0]
         core_logger.info('>>>> training network with %d channel input <<<<'%nchan)
-        core_logger.info('>>>> saving every %d epochs'%save_every)
         core_logger.info('>>>> LR: %0.5f, batch_size: %d, weight_decay: %0.5f'%(self.learning_rate, self.batch_size, weight_decay))
-        core_logger.info('>>>> ntrain = %d'%nimg)
-        core_logger.info('>>>> rescale is %d'%rescale)
+        
         if test_data is not None:
-            core_logger.info('>>>> ntest = %d'%len(test_data))
-        core_logger.info(train_data[0].shape)
+            core_logger.info(f'>>>> ntrain = {nimg}, ntest = {len(test_data)}')
+        else:
+            core_logger.info(f'>>>> ntrain = {nimg}')
         
         tic = time.time()
 
@@ -895,7 +886,8 @@ class UnetModel():
                     LR = np.append(LR, LR[-1]/2 * np.ones(10))
             else:
                 LR = np.append(LR, self.learning_rate*np.ones(max(0,self.n_epochs-10)))
-        
+        else:
+            LR = self.learning_rate * np.ones(self.n_epochs)
 
         lavg, nsum = 0, 0
 
@@ -917,7 +909,10 @@ class UnetModel():
         for iepoch in range(self.n_epochs):
                 
             np.random.seed(iepoch)
-            rperm = np.random.permutation(nimg)
+            if nimg < batch_size:
+                rperm = np.random.choice(nimg, batch_size)
+            else:
+                rperm = np.random.permutation(nimg)
             if SGD:
                 self._set_learning_rate(LR[iepoch])
             
@@ -928,7 +923,6 @@ class UnetModel():
                 imgi, lbl, scale = transforms.random_rotate_and_resize(
                                         [train_data[i] for i in inds], Y=[train_labels[i] for i in inds],
                                         rescale=rsc, scale_range=scale_range, unet=self.unet, inds=inds, omni=self.omni)
-
                 if self.unet and lbl.shape[1]>1 and rescale:
                     lbl[:,1] /= diam_batch[:,np.newaxis,np.newaxis]**2
                 train_loss = self._train_step(imgi, lbl)
@@ -955,10 +949,10 @@ class UnetModel():
                         nsum += len(imgi)
 
                     core_logger.info('Epoch %d, Time %4.1fs, Loss %2.4f, Loss Test %2.4f, LR %2.4f'%
-                            (iepoch, time.time()-tic, lavg, lavgt/nsum, self.optimizer.current_lr))
+                            (iepoch, time.time()-tic, lavg, lavgt/nsum, LR[iepoch]))
                 else:
                     core_logger.info('Epoch %d, Time %4.1fs, Loss %2.4f, LR %2.4f'%
-                            (iepoch, time.time()-tic, lavg, self.optimizer.current_lr))
+                            (iepoch, time.time()-tic, lavg, LR[iepoch]))
                 
                 lavg, nsum = 0, 0
                             
@@ -966,11 +960,17 @@ class UnetModel():
                 if iepoch==self.n_epochs-1 or iepoch%save_every==1:
                     # save model at the end
                     if save_each: #separate files as model progresses 
-                        file_name = '{}_{}_{}_{}'.format(self.net_type, file_label, 
-                                                         d.strftime("%Y_%m_%d_%H_%M_%S.%f"),
-                                                         'epoch_'+str(iepoch)) 
+                        if netstr is None:
+                            file_name = '{}_{}_{}_{}'.format(self.net_type, file_label, 
+                                                             d.strftime("%Y_%m_%d_%H_%M_%S.%f"),
+                                                             'epoch_'+str(iepoch)) 
+                        else:
+                            file_name = '{}_{}'.format(netstr, 'epoch_'+str(iepoch))
                     else:
-                        file_name = '{}_{}_{}'.format(self.net_type, file_label, d.strftime("%Y_%m_%d_%H_%M_%S.%f"))
+                        if netstr is None:
+                            file_name = '{}_{}_{}'.format(self.net_type, file_label, d.strftime("%Y_%m_%d_%H_%M_%S.%f"))
+                        else:
+                            file_name = netstr
                     file_name = os.path.join(file_path, file_name)
                     ksave += 1
                     core_logger.info(f'saving network parameters to {file_name}')
