@@ -6,7 +6,14 @@ import tifffile
 import logging, pathlib, sys
 from pathlib import Path
 
-from . import utils, plot, transforms, omnipose
+from . import utils, plot, transforms
+
+try:
+    from omnipose.utils import format_labels
+    import ncolor
+    OMNI_INSTALLED = True
+except:
+    OMNI_INSTALLED = False
 
 try:
     from PyQt5 import QtGui, QtCore, Qt, QtWidgets
@@ -27,7 +34,6 @@ except:
     SERVER_UPLOAD = False
 
 io_logger = logging.getLogger(__name__)
-io_logger.setLevel(logging.DEBUG)
 
 def logger_setup():
     cp_dir = pathlib.Path.home().joinpath('.cellpose')
@@ -145,8 +151,13 @@ def get_label_files(image_names, mask_filter, imf=None):
     # check for masks
     if os.path.exists(label_names[0] + mask_filter + '.tif'):
         label_names = [label_names[n] + mask_filter + '.tif' for n in range(nimg)]
+    elif os.path.exists(label_names[0] + mask_filter + '.tiff'):
+        label_names = [label_names[n] + mask_filter + '.tiff' for n in range(nimg)]
     elif os.path.exists(label_names[0] + mask_filter + '.png'):
         label_names = [label_names[n] + mask_filter + '.png' for n in range(nimg)]
+    # todo, allow _seg.npy
+    #elif os.path.exists(label_names[0] + '_seg.npy'):
+    #    io_logger.info('labels found as _seg.npy files, converting to tif')
     else:
         raise ValueError('labels not provided with correct --mask_filter')
     if not all([os.path.exists(label) for label in label_names]):
@@ -364,21 +375,29 @@ def save_masks(images, masks, flows, file_names, png=True, tif=False, channels=[
     check_dir(maskdir) 
 
     exts = []
-    if masks.ndim > 2 or masks.max()>2**16-1:
+    if masks.ndim > 2:
         png = False
         tif = True
     if png:    
-        exts.append('.png')
+        if masks.max() < 2**16:
+            masks = masks.astype(np.uint16) 
+            exts.append('.png')
+        else:
+            png = False 
+            tif = True
+            io_logger.warning('found more than 65535 masks in each image, cannot save PNG, saving as TIF')
     if tif:
         exts.append('.tif')
 
     # format_labels will also automatically use lowest bit depth possible
-    masks = omnipose.utils.format_labels(masks) 
+    if OMNI_INSTALLED:
+        masks = format_labels(masks) 
 
     # save masks
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         for ext in exts:
+            
             imsave(os.path.join(maskdir,basename + '_cp_masks' + suffix + ext), masks)
             
     if png and MATPLOTLIB and not min(images.shape) > 3:
@@ -417,11 +436,11 @@ def save_masks(images, masks, flows, file_names, png=True, tif=False, channels=[
         imsave(os.path.join(outlinedir, basename + '_outlines' + suffix + '.png'),  imgout)
     
     # ncolor labels (ready for color map application)
-    if masks.ndim < 3 and save_ncolor:
+    if masks.ndim < 3 and OMNI_INSTALLED and save_ncolor:
         check_dir(ncolordir)
         #convert masks to minimal n-color reresentation 
         imsave(os.path.join(ncolordir, basename + '_cp_ncolor_masks' + suffix + '.png'),
-               omnipose.utils.ncolorlabel(masks))
+               ncolor.label(masks))
     
     # save RGB flow picture
     if masks.ndim < 3 and save_flows:
