@@ -242,7 +242,7 @@ def _true_positive(iou, th):
     tp = match_ok.sum()
     return tp
 
-def flow_error(maski, dP_net, use_gpu=False, device=None, omni=True):
+def flow_error(maski, dP_net, use_gpu=False, device=None):
     """ error in flows from predicted masks vs flows predicted by network run on image
 
     This function serves to benchmark the quality of masks, it works as follows
@@ -275,12 +275,9 @@ def flow_error(maski, dP_net, use_gpu=False, device=None, omni=True):
         print('ERROR: net flow is not same size as predicted masks')
         return
 
-    # ensure unique masks
-    maski = np.reshape(np.unique(maski.astype(np.float32), return_inverse=True)[1], maski.shape)
-
     # flows predicted from estimated masks
     idx = -1 # flows are the last thing returned now
-    dP_masks = dynamics.masks_to_flows(maski, use_gpu=use_gpu, device=device, omni=omni)[idx] 
+    dP_masks = dynamics.masks_to_flows(maski, use_gpu=use_gpu, device=device)[idx] 
     # difference between predicted flows vs mask flows
     flow_errors=np.zeros(maski.max())
     for i in range(dP_masks.shape[0]):
