@@ -3,21 +3,28 @@ from pathlib import Path
 import numpy as np
 from tqdm import trange, tqdm
 from urllib.parse import urlparse
-import torch, gc
-gc.enable()
+import torch
 
 import logging
 models_logger = logging.getLogger(__name__)
-models_logger.setLevel(logging.DEBUG)
 
-from . import transforms, dynamics, utils, plot, omnipose
+from . import transforms, dynamics, utils, plot
 from .core import UnetModel, assign_device, check_mkl, MXNET_ENABLED, parse_model_string
 
+try:
+    import omnipose
+    OMNI_INSTALLED = True
+except:
+    OMNI_INSTALLED = False
+
 _MODEL_URL = 'https://www.cellpose.org/models'
 _MODEL_DIR_ENV = os.environ.get("CELLPOSE_LOCAL_MODELS_PATH")
 _MODEL_DIR_DEFAULT = pathlib.Path.home().joinpath('.cellpose', 'models')
 MODEL_DIR = pathlib.Path(_MODEL_DIR_ENV) if _MODEL_DIR_ENV else _MODEL_DIR_DEFAULT
-MODEL_NAMES = ['cyto','nuclei','bact','cyto2','bact_omni','cyto2_omni']
+if OMNI_INSTALLED:
+    MODEL_NAMES = ['cyto','nuclei','cyto2','bact','bact_omni','cyto2_omni']
+else:
+    MODEL_NAMES = ['cyto','nuclei','cyto2']
 
 def model_path(model_type, model_index, use_torch):
     torch_str = 'torch' if use_torch else ''
@@ -523,9 +530,6 @@ class CellposeModel(UnetModel):
                 
             verbose: bool (optional, default False)
                 turn on additional output to logs for debugging 
-                
-            verbose: bool (optional, default False)
-                turn on additional output to logs for debugging
             
             transparency: bool (optional, default False)
                 modulate flow opacity by magnitude instead of brightness (can use flows on any color background) 
@@ -547,19 +551,21 @@ class CellposeModel(UnetModel):
                 style vector summarizing each image, also used to estimate size of objects in image
 
         """
-        if verbose:
-            models_logger.info('Evaluating with omni %d, cluster %d, flow_threshold %f'%(omni,cluster,flow_threshold))
         
         if cellprob_threshold is not None or dist_threshold is not None:
             mask_threshold = deprecation_warning_cellprob_dist_threshold(cellprob_threshold, dist_threshold)
         
+        if verbose:
+            models_logger.info('Evaluating with flow_threshold %0.2f, mask_threshold %0.2f'%(flow_threshold, mask_threshold))
+            if omni:
+                models_logger.info('using omni model, cluster %d'%(omni,cluster))
+        
         if isinstance(x, list) or x.squeeze().ndim==5:
             masks, styles, flows = [], [], []
             tqdm_out = utils.TqdmToLogger(models_logger, level=logging.INFO)
             nimg = len(x)
             iterator = trange(nimg, file=tqdm_out) if nimg>1 else range(nimg)
             for i in iterator:
-                torch.cuda.empty_cache() #attempt to clear memory before evaluation
                 maski, stylei, flowi = self.eval(x[i], 
                                                  batch_size=batch_size, 
                                                  channels=channels[i] if (len(channels)==len(x) and 
@@ -610,7 +616,7 @@ class CellposeModel(UnetModel):
                 if not self.torch:
                     self.net.collect_params().grad_req = 'null'
             
-            masks, styles, dP, dist, bd, p, tr = self._run_cp(x, 
+            masks, styles, dP, cellprob, p, bd, tr = self._run_cp(x, 
                                                           compute_masks=compute_masks,
                                                           normalize=normalize,
                                                           invert=invert,
@@ -633,24 +639,21 @@ class CellposeModel(UnetModel):
                                                           calc_trace=calc_trace,
                                                           verbose=verbose)
 
-            flows = [plot.dx_to_circ(dP,transparency=transparency,mask=1-1/(1+np.exp(dist))), dP, dist, bd, p, tr]
+            flows = [plot.dx_to_circ(dP,transparency=transparency), dP, cellprob, p, bd, tr]
             return masks, flows, styles
 
     def _run_cp(self, x, compute_masks=True, normalize=True, invert=False,
                 rescale=1.0, net_avg=True, resample=False,
                 augment=False, tile=True, tile_overlap=0.1,
                 mask_threshold=0.0, diam_threshold=12., flow_threshold=0.4, min_size=15,
-                interp=False, cluster=False, anisotropy=1.0, do_3D=False, stitch_threshold=0.0,
+                interp=True, cluster=False, anisotropy=1.0, do_3D=False, stitch_threshold=0.0,
                 omni=False, calc_trace=False, verbose=False):
         
-        # not sure yet if this helps with memeory 
-        gc.collect()
-        torch.cuda.empty_cache()
-        
         tic = time.time()
         shape = x.shape
         nimg = shape[0]        
         
+        bd, tr = None, None
         if do_3D:
             img = np.asarray(x)
             if normalize or invert:
@@ -658,25 +661,21 @@ class CellposeModel(UnetModel):
             yf, styles = self._run_3D(img, rsz=rescale, anisotropy=anisotropy, 
                                       net_avg=net_avg, augment=augment, tile=tile,
                                       tile_overlap=tile_overlap)
-            dist = yf[0][-1] + yf[1][-1] + yf[2][-1] # changed in name only, no edits to 3D yet
+            cellprob = yf[0][-1] + yf[1][-1] + yf[2][-1] 
             dP = np.stack((yf[1][0] + yf[2][0], yf[0][0] + yf[2][1], yf[0][1] + yf[1][1]),
                           axis=0) # (dZ, dY, dX)
-            
-            # just for compatibility below for now
-            bd = np.zeros_like(dist)
-            tr = None
+            del yf
         else:
             tqdm_out = utils.TqdmToLogger(models_logger, level=logging.INFO)
             iterator = trange(nimg, file=tqdm_out) if nimg>1 else range(nimg)
             styles = np.zeros((nimg, self.nbase[-1]), np.float32)
             if resample:
                 dP = np.zeros((2, nimg, shape[1], shape[2]), np.float32)
-                dist = np.zeros((nimg, shape[1], shape[2]), np.float32)
-                bd = np.zeros_like(dist)
+                cellprob = np.zeros((nimg, shape[1], shape[2]), np.float32)
+                
             else:
                 dP = np.zeros((2, nimg, int(shape[1]*rescale), int(shape[2]*rescale)), np.float32)
-                dist = np.zeros((nimg, int(shape[1]*rescale), int(shape[2]*rescale)), np.float32)
-                bd = np.zeros_like(dist)
+                cellprob = np.zeros((nimg, int(shape[1]*rescale), int(shape[2]*rescale)), np.float32)
                 
             for i in iterator:
                 img = np.asarray(x[i])
@@ -691,16 +690,16 @@ class CellposeModel(UnetModel):
                 if resample:
                     yf = transforms.resize_image(yf, shape[1], shape[2])
 
-                dist[i] = yf[:,:,2]
+                cellprob[i] = yf[:,:,2]
                 dP[:, i] = yf[:,:,:2].transpose((2,0,1)) 
                 if self.nclasses == 4:
+                    if i==0:
+                        bd = np.zeros_like(cellprob)
                     bd[i] = yf[:,:,3]
                 styles[i] = style
+            del yf, style
+        styles = styles.squeeze()
         
-        #again, attempt to deal with memory overuse
-        yf, style = None, None
-        gc.collect()
-        torch.cuda.empty_cache()
         
         net_time = time.time() - tic
         if nimg > 1:
@@ -710,31 +709,36 @@ class CellposeModel(UnetModel):
             tic=time.time()
             niter = 200 if do_3D else (1 / rescale * 200)
             if do_3D:
-                masks, p, tr = dynamics.compute_masks(dP, dist, bd, niter=niter, mask_threshold=mask_threshold,
+                masks, p, tr = dynamics.compute_masks(dP, cellprob, bd, niter=niter, mask_threshold=mask_threshold,
                                                       diam_threshold=diam_threshold,flow_threshold=flow_threshold,
                                                       interp=interp, cluster=cluster, do_3D=do_3D, min_size=min_size,
                                                       resize=None, omni=omni, calc_trace=calc_trace, verbose=verbose,
                                                       use_gpu=self.gpu, device=self.device, nclasses=self.nclasses)
             else:
-                masks = np.zeros((nimg, shape[1], shape[2]), np.uint16)
-                p = np.zeros(dP.shape, np.uint16)
-
-                tr = [[]]*nimg # trace may not work correctly with multiple images currently, still need to test it 
+                masks, p, tr = [], [], []
                 resize = [shape[1], shape[2]] if not resample else None
                 for i in iterator:
-                    masks[i], p[:,i], tr[i] = dynamics.compute_masks(dP[:,i], dist[i], bd[i], 
-                                                                     niter=niter, 
-                                                                     mask_threshold=mask_threshold,
-                                                                     flow_threshold=flow_threshold, 
-                                                                     diam_threshold=diam_threshold, 
-                                                                     interp=interp, cluster=cluster,
-                                                                     resize=resize, 
-                                                                     omni=omni, calc_trace=calc_trace, 
-                                                                     verbose=verbose,
-                                                                     use_gpu=self.gpu, 
-                                                                     device=self.device, 
-                                                                     nclasses=self.nclasses)
-            
+                    bdi = bd[i] if bd is not None else None
+                    outputs = dynamics.compute_masks(dP[:,i], cellprob[i], bdi, 
+                                                    niter=niter, 
+                                                    mask_threshold=mask_threshold,
+                                                    flow_threshold=flow_threshold, 
+                                                    diam_threshold=diam_threshold, 
+                                                    interp=interp, cluster=cluster,
+                                                    resize=resize, 
+                                                    omni=omni, calc_trace=calc_trace, 
+                                                    verbose=verbose,
+                                                    use_gpu=self.gpu, 
+                                                    device=self.device, 
+                                                    nclasses=self.nclasses)
+                    masks.append(outputs[0])
+                    p.append(outputs[1])
+                    tr.append(outputs[2])
+                
+                masks = np.array(masks)
+                p = np.array(p)
+                tr = np.array(tr)
+
                 if stitch_threshold > 0 and nimg > 1:
                     models_logger.info(f'stitching {nimg} planes using stitch_threshold={stitch_threshold:0.3f} to make 3D masks')
                     masks = utils.stitch3D(masks, stitch_threshold=stitch_threshold)
@@ -742,15 +746,16 @@ class CellposeModel(UnetModel):
             flow_time = time.time() - tic
             if nimg > 1:
                 models_logger.info('masks created in %2.2fs'%(flow_time))
+            masks, dP, cellprob, p = masks.squeeze(), dP.squeeze(), cellprob.squeeze(), p.squeeze()
+            bd = bd.squeeze() if bd is not None else bd
         else:
             masks, p , tr = np.zeros(0), np.zeros(0), np.zeros(0) #pass back zeros if not compute_masks
-        
-        return masks.squeeze(), styles.squeeze(), dP.squeeze(), dist.squeeze(), bd.squeeze(), p.squeeze(), tr
+        return masks, styles, dP, cellprob, p, bd, tr
 
         
     def loss_fn(self, lbl, y):
         """ loss function between true labels lbl and prediction y """
-        if self.omni:
+        if self.omni and OMNI_INSTALLED:
              #loss function for omnipose field 
             loss = omnipose.core.loss(self, lbl, y)
         else: # original loss function 
@@ -768,8 +773,9 @@ class CellposeModel(UnetModel):
               test_data=None, test_labels=None, test_files=None,
               channels=None, normalize=True, pretrained_model=None, 
               save_path=None, save_every=100, save_each=False,
-              learning_rate=0.2, n_epochs=500, momentum=0.9, SGD=False,
-              weight_decay=0.00001, batch_size=8, rescale=False, omni=False):
+              learning_rate=0.2, n_epochs=500, momentum=0.9, SGD=True,
+              weight_decay=0.00001, batch_size=8, rescale=True, omni=False,
+              netstr=None):
 
         """ train network with images train_data 
         
@@ -819,7 +825,7 @@ class CellposeModel(UnetModel):
 
             weight_decay: float (default, 0.00001)
 
-            SGD: bool (default, False) use SGD as optimization instead of RAdam
+            SGD: bool (default, True) use SGD as optimization instead of RAdam
 
             batch_size: int (optional, default 8)
                 number of 224x224 patches to run simultaneously on the GPU
@@ -830,6 +836,9 @@ class CellposeModel(UnetModel):
                 if True it assumes you will fit a size model after training or resize your images accordingly,
                 if False it will try to train the model to be scale-invariant (works worse)
 
+            netstr: str (default, None)
+                name of network, otherwise saved with name as params + training start time
+
         """
         if rescale:
             models_logger.info(f'Training with rescale = {rescale:.2f}')
@@ -839,15 +848,100 @@ class CellposeModel(UnetModel):
         # check if train_labels have flows
         train_flows = dynamics.labels_to_flows(train_labels, files=train_files, use_gpu=self.gpu, device=self.device, omni=omni)
         if run_test:
-            test_flows = dynamics.labels_to_flows(test_labels, files=test_files)
+            test_flows = dynamics.labels_to_flows(test_labels, files=test_files, use_gpu=self.gpu, device=self.device)
         else:
             test_flows = None
         
         model_path = self._train_net(train_data, train_flows, 
                                      test_data, test_flows,
-                                     pretrained_model, save_path, save_every, save_each,
+                                     save_path, save_every, save_each,
                                      learning_rate, n_epochs, momentum, weight_decay, SGD, 
-                                     batch_size, rescale)
+                                     batch_size, rescale, netstr)
+        self.pretrained_model = model_path
+        return model_path
+
+    def retrain(self, train_data, train_labels, train_files=None, 
+              test_data=None, test_labels=None, test_files=None,
+              channels=None, normalize=True, 
+              save_path=None, save_every=100, save_each=False,
+              learning_rate=0.025, n_epochs=100, momentum=0.9, SGD=True,
+              weight_decay=0.0001, batch_size=8, rescale=True, omni=False, netstr=None):
+
+        """ retrain network with images train_data 
+        
+            Parameters
+            ------------------
+
+            train_data: list of arrays (2D or 3D)
+                images for training
+
+            train_labels: list of arrays (2D or 3D)
+                labels for train_data, where 0=no masks; 1,2,...=mask labels
+                can include flows as additional images
+
+            train_files: list of strings
+                file names for images in train_data (to save flows for future runs)
+
+            test_data: list of arrays (2D or 3D)
+                images for testing
+
+            test_labels: list of arrays (2D or 3D)
+                labels for test_data, where 0=no masks; 1,2,...=mask labels; 
+                can include flows as additional images
+        
+            test_files: list of strings
+                file names for images in test_data (to save flows for future runs)
+
+            channels: list of ints (default, None)
+                channels to use for training
+
+            normalize: bool (default, True)
+                normalize data so 0.0=1st percentile and 1.0=99th percentile of image intensities in each channel
+
+            save_path: string (default, None)
+                where to save trained model, if None it is not saved
+
+            save_every: int (default, 100)
+                save network every [save_every] epochs
+
+            learning_rate: float (default, 0.2)
+                learning rate for training
+
+            n_epochs: int (default, 500)
+                how many times to go through whole training set during training
+
+            weight_decay: float (default, 0.00001)
+
+            SGD: bool (default, True) use SGD as optimization instead of RAdam
+
+            batch_size: int (optional, default 8)
+                number of 224x224 patches to run simultaneously on the GPU
+                (can make smaller or bigger depending on GPU memory usage)
+
+            rescale: bool (default, True)
+                whether or not to rescale images to diam_mean during training, 
+                if True it assumes you will fit a size model after training or resize your images accordingly,
+                if False it will try to train the model to be scale-invariant (works worse)
+
+            netstr: str (default, None)
+                name of network, otherwise saved with name as params + training start time
+
+        """
+        train_data, train_labels, test_data, test_labels, run_test = transforms.reshape_train_test(train_data, train_labels,
+                                                                                                   test_data, test_labels,
+                                                                                                   channels, normalize, omni)
+        # check if train_labels have flows
+        train_flows = dynamics.labels_to_flows(train_labels, files=train_files, use_gpu=self.gpu, device=self.device, omni=omni)
+        if run_test:
+            test_flows = dynamics.labels_to_flows(test_labels, files=test_files, use_gpu=self.gpu, device=self.device)
+        else:
+            test_flows = None
+        
+        model_path = self._train_net(train_data, train_flows, 
+                                     test_data, test_flows,
+                                     save_path, save_every, save_each,
+                                     learning_rate, n_epochs, momentum, weight_decay, SGD, 
+                                     batch_size, rescale, netstr)
         self.pretrained_model = model_path
         return model_path
 
@@ -991,13 +1085,10 @@ class SizeModel():
                              tile=tile,
                              batch_size=batch_size, 
                              net_avg=False,
-                             rescale=(self.diam_mean / diam_style), 
-#                              rescale =  self.diam_mean / diam_style if self.diam_mean>0 else 1, 
-#                              rescale = None, 
+                             rescale =  self.diam_mean / diam_style if self.diam_mean>0 else 1, 
+                             #flow_threshold=0,
                              diameter=None,
-#                              interp=interp,
                              interp=False,
-#                              flow_threshold=0,
                              omni=omni)[0]
         
         # allow backwards compatibility to older scale metric
