@@ -2,7 +2,7 @@ import os
 import numpy as np
 import cv2
 from scipy.ndimage import gaussian_filter
-import scipy
+from . import utils, io, transforms
 
 try:
     import matplotlib
@@ -10,7 +10,6 @@ try:
 except:
     MATPLOTLIB_ENABLED = False
 
-
 try:
     from skimage import color
     from skimage.segmentation import find_boundaries
@@ -18,9 +17,12 @@ try:
 except:
     SKIMAGE_ENABLED = False
 
-
-from . import utils, io, transforms
-from .omnipose.utils import ncolorlabel, sinebow
+try:
+    from omnipose.utils import sinebow
+    import ncolor
+    OMNI_INSTALLED = True
+except:
+    OMNI_INSTALLED = False
 
 # modified to use sinebow color
 def dx_to_circ(dP,transparency=False,mask=None):
@@ -41,7 +43,7 @@ def dx_to_circ(dP,transparency=False,mask=None):
     """
     
     dP = np.array(dP)
-    mag = transforms.normalize99(np.sqrt(np.sum(dP**2,axis=0)),omni=1)
+    mag = np.clip(transforms.normalize99(np.sqrt(np.sum(dP**2,axis=0))), 0, 1.)
     angles = np.arctan2(dP[1], dP[0])+np.pi
     a = 2
     r = ((np.cos(angles)+1)/a)
@@ -58,7 +60,6 @@ def dx_to_circ(dP,transparency=False,mask=None):
         
     return im
 
-
 def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, omni=False, seg_norm=False, bg_color=None):
     """ plot segmentation results (like on website)
     
@@ -114,21 +115,21 @@ def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None, om
     ax.axis('off')
 
     outlines = utils.masks_to_outlines(maski)
-    c = sinebow(5)
-    colors = np.array(list(c.values()))[1:] 
-    
+
     # Image normalization to improve cell visibility under labels
     if seg_norm:
         fg = 1/9
-        p = transforms.normalize99(img0,omni=omni)
+        p = np.clip(transforms.normalize99(img0,omni=omni), 0, 1)
         img1 = p**(np.log(fg)/np.log(np.mean(p[maski>0])))
     else:
         img1 = img0
     
     # the mask_overlay function changes colors (preserves only hue I think). The label2rgb function from
     # skimage.color works really well. 
-    if SKIMAGE_ENABLED:
-        overlay = color.label2rgb(ncolorlabel(maski),img1,colors,bg_label=0,alpha=1/3)
+    if SKIMAGE_ENABLED and OMNI_INSTALLED:
+        c = sinebow(5)
+        colors = np.array(list(c.values()))[1:] 
+        overlay = color.label2rgb(ncolor.label(maski),img1,colors,bg_label=0,alpha=1/3)
         overlay = np.uint8(np.clip(overlay, 0, 1)*255)
         overlay[maski==0] = img1[maski==0] #restore original level to background regions
     else:
@@ -258,7 +259,7 @@ def image_to_rgb(img0, channels=[0,0], omni=False):
         img = img.mean(axis=-1)[:,:,np.newaxis]
     for i in range(img.shape[-1]):
         if np.ptp(img[:,:,i])>0:
-            img[:,:,i] = transforms.normalize99(img[:,:,i],omni=omni)
+            img[:,:,i] = np.clip(transforms.normalize99(img[:,:,i],omni=omni), 0, 1)
             img[:,:,i] = np.clip(img[:,:,i], 0, 1)
     img *= 255
     img = np.uint8(img)
