@@ -1,14 +1,16 @@
 import numpy as np
 import warnings
 import cv2
-import edt
-import fastremap
 
 import logging
 transforms_logger = logging.getLogger(__name__)
-transforms_logger.setLevel(logging.DEBUG)
 
-from . import dynamics, utils, omnipose
+from . import dynamics, utils
+try:
+    import omnipose, edt, fastremap
+    OMNI_INSTALLED = True
+except:
+    OMNI_INSTALLED = False
 
 def _taper_mask(ly=224, lx=224, sig=7.5):
     bsize = max(224, max(ly, lx))
@@ -191,7 +193,7 @@ def make_tiles(imgi, bsize=224, augment=False, tile_overlap=0.1):
 # also turns out previous formulation can give negative numbers, messes up log operations etc. 
 def normalize99(Y,lower=0.01,upper=99.99,omni=False):
     """ normalize image so 0.0 is 0.01st percentile and 1.0 is 99.99th percentile """
-    if omni:
+    if omni and OMNI_INSTALLED:
         X = omnipose.utils.normalize99(Y)
     else:
         X = Y.copy()
@@ -405,7 +407,7 @@ def normalize_img(img, axis=-1, invert=False, omni=False):
     img = np.moveaxis(img, 0, axis)
     return img
 
-def reshape_train_test(train_data, train_labels, test_data, test_labels, channels, normalize, omni=False):
+def reshape_train_test(train_data, train_labels, test_data, test_labels, channels, normalize=True, omni=False):
     """ check sizes and reshape train and test data for training """
     nimg = len(train_data)
     # check that arrays are correct size
@@ -442,7 +444,6 @@ def reshape_train_test(train_data, train_labels, test_data, test_labels, channel
         return
 
     if not run_test:
-        transforms_logger.info('NOTE: test data not provided OR labels incorrect OR not same number of channels as train data')
         test_data, test_labels = None, None
 
     return train_data, train_labels, test_data, test_labels, run_test
@@ -498,7 +499,6 @@ def reshape_and_normalize_data(train_data, test_data=None, channels=None, normal
             if normalize:
                 data[i] = normalize_img(data[i], axis=0, omni=omni)
         nchan = [data[i].shape[0] for i in range(nimg)]
-        transforms_logger.info('%s channels = %d'%(['train', 'test'][test], nchan[0]))
     run_test = True
     return train_data, test_data, run_test
 
@@ -654,25 +654,24 @@ def random_rotate_and_resize(X, Y=None, scale_range=1., gamma_range=0.5, xy = (2
         nimg = len(X)
         inds = np.arange(nimg)
     
-    if omni:
+    if omni and OMNI_INSTALLED:
         return omnipose.core.random_rotate_and_resize(X, Y=Y, scale_range=scale_range, gamma_range=gamma_range, 
                                                  xy=xy, do_flip=do_flip, rescale=rescale, inds=inds)
     else:
         # backwards compatibility; completely 'stock', no gamma augmentation or any other extra frills. 
         # [Y[i][1:] for i in inds] is necessary because the original transform function does not use masks (entry 0). 
         # This used to be done in the original function call. 
-        if Y is not None:
-            Y = [y[1:] for y in Y]
-        return original_random_rotate_and_resize(X, Y=Y, scale_range=scale_range, xy=xy,
+        return original_random_rotate_and_resize(X, Y=[y[1:] for y in Y] if Y is not None else None, 
+                                                 scale_range=scale_range, xy=xy,
                                                  do_flip=do_flip, rescale=rescale, unet=unet)
 
 
 # I have the omni flag here just in case, but it actually does not affect the tests
 def normalize_field(mu,omni=False):
-    if not omni:
-        mu /= (1e-20 + (mu**2).sum(axis=0)**0.5)
-    else:   
+    if omni and OMNI_INSTALLED:
         mu = omnipose.utils.normalize_field(mu) 
+    else:
+        mu /= (1e-20 + (mu**2).sum(axis=0)**0.5)
     return mu
 
 
@@ -733,8 +732,8 @@ def _image_resizer(img, resize=512, to_uint8=False):
     return img
 
 
-def original_random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224),
-                                      do_flip=True, rescale=None, unet=False):
+def original_random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), 
+                             do_flip=True, rescale=None, unet=False):
     """ augmentation by random rotation and resizing
         X and Y are lists or arrays of length nimg, with dims channels x Ly x Lx (channels optional)
         Parameters
@@ -833,4 +832,4 @@ def original_random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224),
                 lbl[n,1] = (-v1 * np.sin(-theta) + v2*np.cos(-theta))
                 lbl[n,2] = (v1 * np.cos(-theta) + v2*np.sin(-theta))
 
-    return imgi, lbl, scale
+    return imgi, lbl, scale
\ No newline at end of file
