@@ -10,7 +10,12 @@ import numpy as np
 import colorsys
 import io
 
-from . import metrics, omnipose
+from . import metrics
+try:
+    import omnipose, ncolor
+    OMNI_INSTALLED = True
+except:
+    OMNI_INSTALLED = False
 
 try:
     from skimage.morphology import remove_small_holes
@@ -366,9 +371,11 @@ def stitch3D(masks, stitch_threshold=0.25):
             masks[i+1] = istitch[masks[i+1]]
     return masks
 
-# merged deiameter functions
+# merged diameter functions
 def diameters(masks, omni=False, dist_threshold=1):
-    if not omni: #original 'equivalent area circle' diameter
+    if omni and OMNI_INSTALLED: #new distance-field-derived diameter (aggrees with cicle but more general)
+        return omnipose.core.diameters(masks), None
+    else: #original 'equivalent area circle' diameter
         _, counts = np.unique(np.int32(masks), return_counts=True)
         counts = counts[1:]
         md = np.median(counts**0.5)
@@ -376,9 +383,6 @@ def diameters(masks, omni=False, dist_threshold=1):
             md = 0
         md /= (np.pi**0.5)/2
         return md, counts**0.5
-    else: #new distance-field-derived diameter (aggrees with cicle but more general)
-        return omnipose.core.diameters(masks), None
-
 
 def radius_distribution(masks, bins):
     unique, counts = np.unique(masks, return_counts=True)
@@ -431,9 +435,11 @@ def fill_holes_and_remove_small_masks(masks, min_size=15, hole_size=3, scale_fac
     
     """
 
-    if masks.ndim==2:
-        masks = omnipose.utils.format_labels(masks, min_area=min_size) # not sure how this works with 3D... tests pass though
-    
+    if masks.ndim==2 and OMNI_INSTALLED:
+        # formatting to integer is critical
+        # need to test how it does with 3D
+        masks = ncolor.format_labels(masks, min_area=min_size)
+        
     hole_size *= scale_factor
         
     if masks.ndim > 3 or masks.ndim < 2:
@@ -447,9 +453,7 @@ def fill_holes_and_remove_small_masks(masks, min_size=15, hole_size=3, scale_fac
             npix = msk.sum()
             if min_size > 0 and npix < min_size:
                 masks[slc][msk] = 0
-            else:   
-                hsz = np.count_nonzero(msk)*hole_size/100 #turn hole size into percentage
-                #eventually the boundary output should be used to properly exclude real holes vs label gaps 
+            elif npix > 0:   
                 if msk.ndim==3:
                     for k in range(msk.shape[0]):
                         # Omnipose version (breaks 3D tests)
@@ -460,7 +464,8 @@ def fill_holes_and_remove_small_masks(masks, min_size=15, hole_size=3, scale_fac
                         msk[k] = binary_fill_holes(msk[k])
 
                 else:          
-                    if SKIMAGE_ENABLED: # Omnipose version (passes 2D tests)
+                    if OMNI_INSTALLED and SKIMAGE_ENABLED: # Omnipose version (passes 2D tests)
+                        hsz = np.count_nonzero(msk)*hole_size/100 #turn hole size into percentage
                         padmsk = remove_small_holes(np.pad(msk,1,mode='constant'),hsz)
                         msk = padmsk[1:-1,1:-1]
                     else: #Cellpose version
